package net.cattaka.learnukiukiweb.client;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

import net.cattaka.learnukiukiweb.LearnUkiukiWeb;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.net.Uri;
import android.os.Build;
import android.util.Log;

import com.google.android.maps.GeoPoint;

public class UkiukiClient {
	private String model = Uri.encode(Build.MODEL+"__"+ Build.VERSION.RELEASE);

	/**
	 * 通常コンテンツ取得‎V4を利用して感情玉を取得します。
	 * 
	 * @see http://sites.google.com/site/uki2view/uki2server/api/tsuujou-kontentsu-shutoku/v4
	 * @param apiKey APIキー
	 * @param geoPoint 取得する座標
	 * @return 感情玉のリスト
	 */
	public List<UkiukiBallInfo> getUkiukiBallInfoList(String apiKey, GeoPoint geoPoint) {
		HttpClient httpClient = new DefaultHttpClient();  
		HttpParams httpParams = httpClient.getParams();  
		HttpConnectionParams.setConnectionTimeout(httpParams, 30000);  
		HttpConnectionParams.setSoTimeout(httpParams, 30000);

		// コンテンツ取得処理
		DecimalFormat df = new DecimalFormat("0.000000");
		String lat = df.format((double)geoPoint.getLatitudeE6() / 1E6d);
		String lng = df.format((double)geoPoint.getLongitudeE6() / 1E6d);
		
		// HTTP通信用のURLを作成する
		String url = "https://www.uki2view.jp/uki2/message/message/v4/"
			+ "?apikey=" + apiKey
			+ "&lat=" + lat
			+ "&lng=" + lng
			+ "&range=7"
			+ "&sort=1"
			+ "&term=full"
			+ "&model=" + model;

		// HTTP通信を行ってjsonの文字列を取得する
		String jsonString = null;
		try {
			HttpGet objGet = new HttpGet(url);
			HttpResponse objResponse = httpClient.execute(objGet);
			if (objResponse.getStatusLine().getStatusCode() < 400) {
				StringBuilder sb = new StringBuilder(2<<12);
				Reader in = new InputStreamReader(objResponse.getEntity().getContent());
				char[] buf = new char[2 << 12];
				int r;
				while ((r = in.read(buf, 0, buf.length)) != -1) {
					sb.append(buf, 0, r);
				}
				in.close();
				jsonString = sb.toString();
			}
		} catch (IOException e) {
			Log.d(LearnUkiukiWeb.TAG, e.getMessage(), e);
		}
		
		// 取得できてない場合はエラー
		if (jsonString == null) {
			return null;
		}
		
		// 取得結果をJSONに変換する
		List<UkiukiBallInfo> ukiukiBallInfoList = new ArrayList<UkiukiBallInfo>();
		try {
			JSONObject jsonObject = new JSONObject(jsonString);
			JSONObject results = jsonObject.getJSONObject("results");
			JSONArray messageArray = results.getJSONArray("message");
			
			for (int i=0;i<messageArray.length();i++) {
				JSONObject message = messageArray.getJSONObject(i);
				UkiukiBallInfo ubInfo = new UkiukiBallInfo();
				ubInfo.setStatus(message.getInt("status"));
				ubInfo.setLanguage(message.getString("language"));
				ubInfo.setObjectid(message.getString("objectid"));
				ubInfo.setNickname(message.getString("nickname"));
				ubInfo.setLanguage(message.getString("language"));
				ubInfo.setTitle(message.getString("title"));
				ubInfo.setRegisttime(message.getString("registtime"));
				ubInfo.setUserid(message.getString("userid"));
				ubInfo.setUri(message.getString("uri"));
				ubInfo.setMtype(message.getString("mtype"));
				ubInfo.setParentid(message.getString("parentid"));
				ubInfo.setModerate(message.getInt("moderate"));

				int ubLat = (int)(message.getDouble("lat")*1E6);
				int ubLng = (int)(message.getDouble("lng")*1E6);
				ubInfo.setGeoPoint(new GeoPoint(ubLat, ubLng));
				ukiukiBallInfoList.add(ubInfo);
			}
		} catch (JSONException e) {
			Log.d(LearnUkiukiWeb.TAG, e.getMessage(), e);
		}
		
		// 結果を返す
		return ukiukiBallInfoList;
	}
}
